#!/bin/bash

set -eu

cur=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
source $cur/../_utils/test_prepare
WORK_DIR=$TEST_DIR/$TEST_NAME

function run() {
    run_sql_file $cur/data/db1.prepare.sql $MYSQL_HOST1 $MYSQL_PORT1 $MYSQL_PASSWORD1
    run_sql_file $cur/data/db2.prepare.sql $MYSQL_HOST2 $MYSQL_PORT2 $MYSQL_PASSWORD2

    run_dm_worker $WORK_DIR/worker1 $WORKER1_PORT $cur/conf/dm-worker1.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER1_PORT
    run_dm_worker $WORK_DIR/worker2 $WORKER2_PORT $cur/conf/dm-worker2.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER2_PORT
    run_dm_master $WORK_DIR/master $MASTER_PORT $cur/conf/dm-master.toml
    check_rpc_alive $cur/../bin/check_master_online 127.0.0.1:$MASTER_PORT

    # start DM task only
    dmctl_start_task

    # use sync_diff_inspector to check full dump loader
    check_sync_diff $WORK_DIR $cur/conf/diff_config.toml

    run_sql_file $cur/data/db1.increment.sql $MYSQL_HOST1 $MYSQL_PORT1 $MYSQL_PASSWORD1
    run_sql_file $cur/data/db2.increment.sql $MYSQL_HOST2 $MYSQL_PORT2 $MYSQL_PASSWORD2

    # use sync_diff_inspector to check data now!
    check_sync_diff $WORK_DIR $cur/conf/diff_config.toml

    kill_dm_worker
    check_port_offline $WORKER1_PORT 20
    check_port_offline $WORKER2_PORT 20

    export GO_FAILPOINTS='github.com/pingcap/dm/syncer/ExitAfterDDLBeforeFlush=return(true)'
    run_sql_file $cur/data/db1.increment2.sql $MYSQL_HOST1 $MYSQL_PORT1 $MYSQL_PASSWORD1
    run_sql_file $cur/data/db2.increment2.sql $MYSQL_HOST2 $MYSQL_PORT2 $MYSQL_PASSWORD2

    run_dm_worker $WORK_DIR/worker1 $WORKER1_PORT $cur/conf/dm-worker1.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER1_PORT
    run_dm_worker $WORK_DIR/worker2 $WORKER2_PORT $cur/conf/dm-worker2.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER2_PORT
    sleep 1

    check_port_offline $WORKER1_PORT 20
    check_port_offline $WORKER2_PORT 20

    export GO_FAILPOINTS=''

    run_dm_worker $WORK_DIR/worker1 $WORKER1_PORT $cur/conf/dm-worker1.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER1_PORT
    run_dm_worker $WORK_DIR/worker2 $WORKER2_PORT $cur/conf/dm-worker2.toml
    check_rpc_alive $cur/../bin/check_worker_online 127.0.0.1:$WORKER2_PORT
    sleep 1

    # use sync_diff_inspector to check data now!
    echo "check sync diff for the increment replication"
    check_sync_diff $WORK_DIR $cur/conf/diff_config.toml

    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "query-status sequence_sharding" \
        "\"stage\": \"Running\"" 4 \
        "\"unit\": \"Sync\"" 2


    run_sql_file $cur/data/db1.increment3.sql $MYSQL_HOST1 $MYSQL_PORT1 $MYSQL_PASSWORD1
    run_sql_file $cur/data/db2.increment3.sql $MYSQL_HOST2 $MYSQL_PORT2 $MYSQL_PASSWORD2

    sleep 2

    # the first ddl success while the second is conflict
    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "query-status sequence_sharding" \
        "detect inconsistent DDL sequence" 2

    # check no auto resume
    sleep 3
    check_log_contains $WORK_DIR/worker1/log/dm-worker.log "task can't auto resume"

    # resume manually
    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "resume-task sequence_sharding" \
        "\"result\": true" 3

    sleep 2

    # still conflict
    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "query-status sequence_sharding" \
        "detect inconsistent DDL sequence" 2

    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "stop-task sequence_sharding" \
        "\"result\": true" 3
    
    # now upstream schema is conflict, ignore it and restart task
    cp $cur/conf/dm-task.yaml $WORK_DIR/task.yaml
    echo "ignore-checking-items: [\"all\"]" >> $WORK_DIR/task.yaml
    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "start-task $WORK_DIR/task.yaml" \
        "\"result\": true" 3
    
    sleep 2

    # still conflict
    run_dm_ctl $WORK_DIR "127.0.0.1:$MASTER_PORT" \
        "query-status sequence_sharding" \
        "detect inconsistent DDL sequence" 2
}

cleanup_data sharding_target2
# also cleanup dm processes in case of last run failed
cleanup_process $*
run $*
cleanup_process $*

echo "[$(date)] <<<<<< test case $TEST_NAME success! >>>>>>"
